@@ -4,8 +4,8 @@ from typing import Dict
 from typing import List
 from typing import Union
 from polygenic.lib import mobigen_utils
-from polygenic.lib.data_access.dto import SnpData
-from polygenic.lib.data_access.dto import SnpDataManySamples
+from polygenic.data.snp_data import SnpData
+from polygenic.data.snp_data import SnpDataManySamples
 from polygenic.data.vcf_record import VcfRecord
 
 # rsidx
